@@ -31,10 +31,11 @@
 #include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <string.h>
 
+#include <glib/gi18n.h>
 #include <glade/glade-xml.h>
-#include <libgnome/libgnome.h>
 
 #include "gedit2.h"
 #include "gedit-mdi.h"
@@ -52,9 +53,6 @@ struct _GeditDialogGotoLine {
 	GtkWidget *entry;
 };
 
-static void goto_button_pressed (GeditDialogGotoLine * dialog);
-static GeditDialogGotoLine *dialog_goto_line_get_dialog (void);
-static void dialog_destroyed (GtkObject *obj,  void **dialog_pointer);
 
 static void
 dialog_destroyed (GtkObject *obj,  void **dialog_pointer)
@@ -68,6 +66,32 @@ dialog_destroyed (GtkObject *obj,  void **dialog_pointer)
 	}	
 }
 
+static void
+goto_button_pressed (GeditDialogGotoLine *dialog)
+{
+	const gchar *text;
+	GeditView *active_view;
+	GeditDocument *active_document;
+
+	gedit_debug (DEBUG_SEARCH, "");
+
+	active_view = GEDIT_VIEW (bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)));
+	g_return_if_fail (active_view);
+
+	active_document = gedit_view_get_document (active_view);
+	g_return_if_fail (active_document);
+
+	text = gtk_entry_get_text (GTK_ENTRY (dialog->entry));
+
+	if (text != NULL && text[0] != 0)
+	{
+		guint line = MAX (atoi (text) - 1, 0);		
+		gedit_document_goto_line (active_document, line);
+		gedit_view_scroll_to_cursor (active_view);
+		gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	}
+}
+
 static void
 dialog_response_handler (GtkDialog *dlg, gint res_id,  GeditDialogGotoLine *dialog)
 {
@@ -161,7 +185,7 @@ dialog_goto_line_get_dialog (void)
 						      window,
 						      GTK_DIALOG_DESTROY_WITH_PARENT,
 						      GTK_STOCK_CLOSE,
-						      GTK_RESPONSE_CLOSE,
+						      GTK_RESPONSE_CANCEL,
 						      NULL);
 
 	g_return_val_if_fail (dialog->dialog != NULL, NULL);
@@ -230,29 +254,3 @@ gedit_dialog_goto_line (void)
 		gtk_widget_show (dialog->dialog);
 }
 
-static void
-goto_button_pressed (GeditDialogGotoLine *dialog)
-{
-	const gchar* text;
-	GeditView* active_view;
-	GeditDocument* active_document;
-
-	gedit_debug (DEBUG_SEARCH, "");
-
-	active_view = GEDIT_VIEW (bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)));
-	g_return_if_fail (active_view);
-
-	active_document = gedit_view_get_document (active_view);
-	g_return_if_fail (active_document);
-
-	text = gtk_entry_get_text (GTK_ENTRY (dialog->entry));
-	
-	if (text != NULL && text [0] != 0)
-	{
-		guint line = MAX (atoi (text)- 1, 0);		
-		gedit_document_goto_line (active_document, line);
-		gedit_view_scroll_to_cursor (active_view);
-		gtk_widget_grab_focus (GTK_WIDGET (active_view));
-	}
-}
-
